from typing import Type, List, Generic, Dict, Any, TypeVar

from sqlite_dao_ext.sqlite_client.sqlite_query import SqliteQuery
from sqlite_dao_ext.sqlite_client.sqlite_client import SqliteClient
from sqlite_dao_ext.sqlite_dao.sqlite_data_object import SqliteDataObject
from sqlite_dao_ext.sqlite_dao.sqlite_query_generator import SqliteQueryGenerator

SDO = TypeVar("SDO", bound=SqliteDataObject)


class SqliteDao(Generic[SDO]):
    _sqlite_client: SqliteClient
    _data_type: Type[SDO]
    _sql_generator: SqliteQueryGenerator
    _table_name: str

    def __init__(self, sqlite_client: SqliteClient, data_type: Type[SDO]):
        self._sqlite_client = sqlite_client
        self._data_type = data_type
        self._table_name = data_type.table_name()
        self._sql_generator = SqliteQueryGenerator(clz=self._data_type)
        self.create_table()

    def create_table(self):
        self._sqlite_client.execute(*self._sql_generator.create_table_sql)

    def drop_table(self):
        self._sqlite_client.execute(self._sql_generator.drop_table_sql)

    def insert(self, objs: List[SDO]):
        return self._sqlite_client.batch_insert(
            self._sql_generator.insert_sql(objs=objs)
        )

    def insert_or_replace(self, objs: List[SDO]):
        return self._sqlite_client.batch_insert(
            self._sql_generator.insert_sql(objs=objs, supplement="OR REPLACE")
        )

    def insert_or_ignore(self, objs: List[SDO]):
        return self._sqlite_client.batch_insert(
            self._sql_generator.insert_sql(objs=objs, supplement="OR IGNORE")
        )

    def delete_by_values(self, value_map: Dict[str, Any] = None):
        return self._sqlite_client.execute(
            self._sql_generator.delete_by_values_sql(value_map or {}),
        )

    def count_by_values(self, value_map: Dict[str, Any] = None) -> int:
        return self._sqlite_client.execute(
            self._sql_generator.count_by_values_sql(value_map or {}),
            commit=False,
        )[0][0]["count"]

    def query_by_values(
        self, value_map: Dict[str, Any] = None, limit: int = 100, offset: int = 0
    ) -> List[SDO]:
        return list(
            map(
                self._data_type.loads,
                self._sqlite_client.execute(
                    self._sql_generator.query_by_values_sql(
                        value_map or {}, limit=limit, offset=offset
                    ),
                    commit=False,
                )[0],
            )
        )

    def execute(self, *queries: SqliteQuery, commit: bool = True):
        return self._sqlite_client.execute(*queries, commit=commit)

    def __str__(self):
        return f"SqliteDao<{self._data_type.__name__}>(sqlite_client={self._sqlite_client})"

    def __repr__(self):
        return str(self)
